
/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/

//
// DODS-netCDF surrogate library for data retrieval. 
// 
// ReZa 10/14/94

/* $Log: Dputgetg.cc,v $
/* Revision 1.7  1998/02/05 20:14:45  jimg
/* DODS now compiles with gcc 2.8.x
/*
 * Revision 1.6  1996/09/17 17:06:11  jimg
 * Merge the release-2-0 tagged files (which were off on a branch) back into
 * the trunk revision.
 *
 * Revision 1.5.4.2  1996/09/17 00:26:12  jimg
 * Merged changes from a side branch which contained various changes from
 * Reza and Charles.
 * Removed ncdump and netexec since ncdump is now in its own directory and
 * netexec is no longer used.
 *
 * Revision 1.5.4.1  1996/06/25 22:04:13  jimg
 * Version 2.0 from Reza.
 *
// Revision 1.5  1995/07/09  21:33:36  jimg
// Added copyright notice.
//
// Revision 1.4  1994/12/22  04:45:35  reza
// Rearranged for simpler DODS data transmission code interface.
//
// Revision 1.3  1994/12/08  19:01:35  reza
// Added support for local netCDF file access through the original netCDF code.
//
// Revision 1.2  1994/11/23  21:06:02  reza
// First working version.
//
// Revision 1.1  1994/11/03  04:37:59  reza
// Preliminary version of netCDF -> DODS library.
// */

#include "config_nc.h"

static char rcsid[] __unused__ ={"$Id: Dputgetg.cc,v 1.7 1998/02/05 20:14:45 jimg Exp $"};

#include <assert.h>
#include "Dnetcdf.h"

extern int
DODvario(int cdfid,int varid,const long *start,const long *edges,
	 const long *stride, void *values);

extern void
var_info(int cdfid,Pix p, nc_type *typep,int *ndimsp, int dims[]);


int
GenRead(int cdfid, int varid, const long *start, const long *count,
	const long *stride, const long *imap, void *values)
{

    if ((*conns)[cdfid] == NULL) // does the file id exist?
      return(-1) ;

    if (( varid < 0) || (varid > (*conns)[cdfid]->nvars()))
      {
	NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	return(-1) ;
      }


    // is memory arrangment required?
    if (imap == NULL) {
      
      return DODvario(cdfid, varid, start, count, stride, values);
    }
    else {
      
      //  The variable is an array with IMAP.    

      int	maxidim;	         /* maximum dimensional index */
      int       shape[MAX_VAR_DIMS];    /* Variable shape */
      int	idim, ndim;            /* dim. ID and number of dim. */
      char	*valp	= (char*)values;
      long	mycount[MAX_NC_DIMS];
      long	mystart[MAX_NC_DIMS];
      long	mystride[MAX_NC_DIMS];
      long	myimap[MAX_NC_DIMS];
      long	stop[MAX_NC_DIMS];	 /* stop indexes */
      long	length[MAX_NC_DIMS];	/* edge lengths in bytes */
      nc_type   datatype;              /* Variable type */

      //select the Pix for the given varid
      Pix q = (*conns)[cdfid]->dds().first_var();
      for (int i=0; i < varid ; i++)
	(*conns)[cdfid]->dds().next_var(q);
      
      //get the variable shape and type
      var_info(cdfid, q, &datatype, &ndim, shape);
      maxidim = ndim - 1;

      // sizeof each value 
      int szof = nctypelen(datatype);  

      // scalar variable with an imap?
      if (maxidim < 0) {
	NCadvise(NC_EINVAL, "Scalar with index mapping");
	return(-1) ;
      }
     
      // Verify stride argument.
      for (idim = 0; idim < ndim; ++idim) {
	if (stride != NULL && stride[idim] < 1) {
	  NCadvise(NC_EINVAL, "Non-positive stride");
	  return(-1) ;
	}
      }
      
      // Initialize I/O parameters (set defaults).
      long nels = 1;
      for (idim = maxidim; idim >= 0; --idim) {
	mystart[idim]	= start != NULL
	  ? start[idim]
	  : 0;
	mycount[idim]	= count != NULL
	  ? count[idim]
	  : shape[idim] - mystart[idim]; 
	mystride[idim]	= stride != NULL 
	  ? stride[idim]
	  : 1;
	myimap[idim]	= imap != NULL 
	  ? imap[idim]
	  : idim == maxidim
	  ? szof 
	  : myimap[idim+1] * mycount[idim+1];
	
	nels          = mycount[idim]*nels;
	length[idim]  = myimap[idim] * mycount[idim];
	stop[idim]    = mystart[idim] + mycount[idim];
      }
      
      /*
       * As an optimization, adjust I/O parameters when the fastest 
       * dimension has unity stride both externally and internally.
       * In this case, the user could have called a simpler routine
       * (i.e. ncvarget() or ncvarput()).
       */
      int iocount = 1;
      if (mystride[maxidim] == 1 && myimap[maxidim] == szof) {
	iocount             = mycount[maxidim];
	mystride[maxidim]   = mycount[maxidim];	
	myimap[maxidim]     = length[maxidim];
      }
      
      // read only once over the net into the temp. buffer
      char *valpBuf = new char [(nels*szof)];
      
      int iostat = DODvario(cdfid, varid, mystart, mycount, mystride, 
			    (void *)valpBuf);
      if (iostat != 0)
	return iostat;
      
      // Perform IMAP in the buffer.  Exit when done.
      for (;;) {
	
	// calculate the memory offset
	int offset = 0;
	for (idim = maxidim; idim >= 0; --idim){
	  offset += idim == maxidim 
	    ? mystart[idim] * szof 
	    : mystart[idim] * length[idim+1];
	}

	memcpy((void *)valp, (void *)(valpBuf+offset), (size_t)szof*iocount);
	
	/*
	 * The following code permutes through the variable's external
	 * start-index space and it's internal address space.  At the 
	 * UPC, this algorithm is commonly called `odometer code'.
	 */
	idim = maxidim;
      carry:
	valp		+= myimap[idim];
	mystart[idim]	+= 1;	    
	if (mystart[idim] >= stop[idim]) {
	  mystart[idim]	 = start[idim];
	  valp		-= length[idim];
	  if (--idim < 0){
	    delete [] valpBuf;    /* remove temp buffer for imap */	    
	    return 0;
	  }
	  goto carry;
	}
      }				/* IMAP loop */
    }			          /* variable is an array wiht imap*/
}


int
ncvargetg(int cdfid, int varid, const long *start, const long *count,
	  const long *stride, const long *imap, ncvoid *values)
{
if((*conns)[cdfid] == NULL) // does the file id exist?
    return(-1) ;

#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lncvargetg((*conns)[cdfid]->ncid(),varid, start, count,
          stride, imap, values);
    return rcode;
  }
#endif
  cdf_routine_name = "ncvargetg" ;

  return GenRead(cdfid, varid, start, count, 
		 stride, imap, (void *)values);
}


int
ncvargets(int cdfid, int varid, const long *start, const long *count,
	  const long *stride, ncvoid *values)
{
if((*conns)[cdfid] == NULL) // does the file id exist?
    return(-1) ;

#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lncvargets((*conns)[cdfid]->ncid(),varid, start, count,
          stride, values);
    return rcode;
  }
#endif
  cdf_routine_name = "ncvargets" ;

  return GenRead(cdfid, varid, start, count, 
		 stride, (long*)0, (void *)values);
}


int
ncvarputg(int cdfid, int varid, const long *start, const long *count, 
	  const long *stride, const long *imap, const ncvoid *values)
{
if((*conns)[cdfid] == NULL) // does the file id exist?
    return(-1) ;

#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lncvarputg((*conns)[cdfid]->ncid(),varid, start, count,
          stride, imap, values);
    return rcode;
  }
#endif
  cdf_routine_name = "ncvarputg" ;
  NCadvise(NC_EPERM, "DODS network interface is read-only") ;
  return(-1);
}


int
ncvarputs(int cdfid, int varid, const long *start, const long *count, 
	  const long *stride, const ncvoid *values)
{
if((*conns)[cdfid] == NULL) // does the file id exist?
    return(-1) ;

#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lncvarputs((*conns)[cdfid]->ncid(),varid, start, count,
          stride, values);
    return rcode;
  }
#endif
  cdf_routine_name = "ncvarputs" ;
  NCadvise(NC_EPERM, "DODS network interface is read-only") ;
  return(-1);
}

