
#include "../ncview.includes.h"
#include "../ncview.defines.h"

#define DEFAULT_PLOT_LOCK_STATE True

extern Widget topLevel;

static Widget
	plot_XY_popup_widget[MAX_PLOT_XY],
		plot_XY_canvas_widget[MAX_PLOT_XY],
		plot_close_button_widget[MAX_PLOT_XY],
		plot_print_button_widget[MAX_PLOT_XY],
		plot_locked_button_widget[MAX_PLOT_XY],
		plot_XY_widget[MAX_PLOT_XY];

static int	plot_XY_line[MAX_PLOT_XY][MAX_LINES_PER_PLOT],
		plot_XY_locked[MAX_PLOT_XY],
		n_lines_on_plot[MAX_PLOT_XY];

static int	pXY_color[MAX_LINES_PER_PLOT],
		pXY_pstyle[MAX_LINES_PER_PLOT] = {XtMARKER_DIAMOND,
						  XtMARKER_FCIRCLE,
						  XtMARKER_BOWTIE,
						  XtMARKER_DTRIANGLE,
						  XtMARKER_FLTRIANGLE},
		pXY_lstyle[MAX_LINES_PER_PLOT] = {XtLINE_SOLID,
						  XtLINE_SOLID,
						  XtLINE_DOTTED,
						  XtLINE_SOLID,
						  XtLINE_DOTTED };

static char	*pXY_colorname[MAX_LINES_PER_PLOT] = {"red", "white", "blue",
					"green", "black" };

static int	n_plot_xy = 0;

void 	plot_XY_close_callback(Widget w, XtPointer client_data, XtPointer call_data);
void 	plot_XY_print_callback(Widget w, XtPointer client_data, XtPointer call_data);
void 	plot_XY_locked_callback(Widget w, XtPointer client_data, XtPointer call_data);
void 	x_popup_XY_graph( long n, float *xvals, float *yvals, char *xtitle, char *ytitle,
	char *title, char *legend );

static int last_popup_x = 18, last_popup_y = 18;

/***********************************************************************************/

	void
plot_xy_init()
{
	int	i;

	/* Mark the Plot-XY widgets as unused */
	for( i=0; i<MAX_PLOT_XY; i++ ) {
		plot_XY_popup_widget[i] = (Widget)(-1);
		n_lines_on_plot[i]      = 0;
		plot_XY_locked[i]       = FALSE;
		}
}

	void
x_popup_XY_graph( long n, float *xvals, float *yvals, char *x_axis_title,
	char *y_axis_title, char *title, char *legend )
{
	int	i, index, color_id;

	if( (index = locked_plot()) != -1 ) {
		/* Add a line to an already-existing plot */

		n_lines_on_plot[index]++;
		plot_XY_line[index][n_lines_on_plot[index]] = SciPlotListCreateFromFloat( 
				plot_XY_widget[index], n, xvals, 
				yvals, legend );
		color_id = SciPlotAllocNamedColor( 
				plot_XY_widget[index], 
				pXY_colorname[n_lines_on_plot[index]] );
		SciPlotListSetStyle( 
				plot_XY_widget[index], 
				plot_XY_line[index][n_lines_on_plot[index]],
				color_id,
				pXY_pstyle[n_lines_on_plot[index]], 
				color_id,
				pXY_lstyle[n_lines_on_plot[index]] );
		SciPlotUpdate( plot_XY_widget[index] ); 
		if( n_lines_on_plot[index] >= (MAX_LINES_PER_PLOT-1) ) {
			plot_XY_locked[index] = FALSE;
			XtVaSetValues( plot_locked_button_widget[index], XtNstate, FALSE, NULL );
			}
		}
	else
		{
		/* Create a new plot! */

		if( n_plot_xy == MAX_PLOT_XY ) {
			x_error( "Reached maximum # of XY plots!" );
			return;
			}
		
		index = get_free_plot_xy_index();
		n_lines_on_plot[index] = 0;

		plot_XY_popup_widget[index] = XtVaCreatePopupShell(
			title,
			transientShellWidgetClass,
			topLevel,
			NULL );
	
		plot_XY_canvas_widget[index] = XtVaCreateManagedWidget(
			"PlotXY_canvas",
			formWidgetClass,
			plot_XY_popup_widget[index],
			XtNborderWidth, 0,
			NULL);
	
		plot_close_button_widget[index] = XtVaCreateManagedWidget(
			"Close",
			commandWidgetClass,
			plot_XY_canvas_widget[index],
			NULL);
	
		plot_print_button_widget[index] = XtVaCreateManagedWidget(
			"Print",
			commandWidgetClass,
			plot_XY_canvas_widget[index],
			XtNfromHoriz, 	plot_close_button_widget[index],
			NULL);
	
       	 	plot_locked_button_widget[index] = XtVaCreateManagedWidget(
       	         	"Locked",
       	         	toggleWidgetClass,
       	         	plot_XY_canvas_widget[index],
			XtNfromHoriz, 	plot_print_button_widget[index],
			XtNstate,	DEFAULT_PLOT_LOCK_STATE,
       		        NULL);

       	 	XtAddCallback( plot_close_button_widget[index],     XtNcallback, 
				plot_XY_close_callback, (XtPointer)MESSAGE_OK);

       	 	XtAddCallback( plot_print_button_widget[index],     XtNcallback, 
					plot_XY_print_callback, (XtPointer)MESSAGE_OK);

       	 	XtAddCallback( plot_locked_button_widget[index],     XtNcallback, 
				plot_XY_locked_callback, (XtPointer)MESSAGE_OK);

		plot_XY_widget[index] = XtVaCreateManagedWidget(
			"plotXY",
			sciplotWidgetClass,
			plot_XY_canvas_widget[index],
			XtNheight, 	(XtArgVal)300,
			XtNwidth, 	(XtArgVal)650,
			XtNxLabel, 	x_axis_title,
			XtNyLabel, 	y_axis_title,
       	         	XtNxLog,        False,
			XtNplotTitle,	title,
       	         	XtNtop,         XtChainTop,
       	         	XtNleft,        XtChainLeft,
			XtNshowLegend,	True,
			XtNshowTitle,	True,
       	         	XtNright,       XtChainRight,
			XtNfromVert, 	plot_close_button_widget[index],
       		        NULL);

		plot_XY_line[index][0] = SciPlotListCreateFromFloat( 
				plot_XY_widget[index], 
				n, xvals, yvals, legend );

		color_id = SciPlotAllocNamedColor( 
				plot_XY_widget[index], 
				pXY_colorname[0] );

		SciPlotListSetStyle( 
				plot_XY_widget[index], 
				plot_XY_line[index][0],
				color_id,
				pXY_pstyle[0],
				color_id,
				pXY_lstyle[0] );
		SciPlotUpdate( plot_XY_widget[index] ); 

		/* Move the widget to a place which does not cover the
		 * last popped-up widget and pop it up.
		 */
		XtVaSetValues( plot_XY_popup_widget[index], 
			XtNx, last_popup_x, XtNy, last_popup_y, NULL );
		XtPopup( plot_XY_popup_widget[index], XtGrabNone );

		/* Move the popup place so the next one won't cover this one. */
		last_popup_x += 10;
		last_popup_y += 10;

		plot_XY_locked[index] = DEFAULT_PLOT_LOCK_STATE;

		n_plot_xy++;
		}
}

	int
get_free_plot_xy_index()
{
	int	i;

	for( i=0; i<MAX_PLOT_XY; i++ )
		if( plot_XY_popup_widget[i] == (Widget)(-1) )
			return( i );

	fprintf( stderr, "Internal error -- can't find free XY plot index!\n" );
	exit( -1 );
	/* Make compiler error checkers happy */
	return( 0 );
}

        void
plot_XY_locked_callback(Widget widget, XtPointer client_data, XtPointer call_data)
{
	int	i, j;

	i = 0;
	while( (plot_locked_button_widget[i] != widget) && (i < MAX_PLOT_XY) )
		i++;
	if( i == MAX_PLOT_XY ) {
		fprintf( stderr, "Internal error!  Can't find widget to change lock on!\n" );
		exit( -1 );
		}

	/* We can't deliberately lock this plot if it already
	 * has the maximum number of lines it can hold.
	 */
	if( n_lines_on_plot[i] == (MAX_LINES_PER_PLOT-1) ) {
		x_error( "You can't lock a plot which already\nhas the maximum number of plots!\n" );
		XtVaSetValues( plot_locked_button_widget[i], XtNstate, FALSE, NULL );
		return;
		}

	if( plot_XY_locked[i] == TRUE )
		plot_XY_locked[i] = FALSE;
	else
		{
		/* We can only have one locked plot at a time, so 
		 * if this one is to be locked, make sure none of
		 * the others are!
		 */
		for( j=0; j<MAX_PLOT_XY; j++ )
			plot_XY_locked[j] = FALSE;
		plot_XY_locked[i] = TRUE;
		}
}

        void
plot_XY_print_callback(Widget widget, XtPointer client_data, XtPointer call_data)
{
	int	i, message;
	char	filename[256];

	strcpy( filename, "ncview.ps" );
	i = 0;
	while( (plot_print_button_widget[i] != widget) && (i < MAX_PLOT_XY) )
		i++;
	if( i == MAX_PLOT_XY ) {
		fprintf( stderr, "Internal error!  Can't find widget to print from!\n" );
		exit( -1 );
		}

	message = x_dialog( "File to print to:", filename, TRUE );
	if( message != MESSAGE_OK )
		return;

	SciPlotPSCreate( plot_XY_widget[i], filename );
}

        void
plot_XY_close_callback(Widget widget, XtPointer client_data, XtPointer call_data)
{
	int	i;

	i = 0;
	while( (plot_close_button_widget[i] != widget) && (i < MAX_PLOT_XY) )
		i++;
	if( i == MAX_PLOT_XY ) {
		fprintf( stderr, "Internal error!  Can't find widget to close XY plot!\n" );
		exit( -1 );
		}

	XtDestroyWidget( plot_XY_popup_widget[i] );

	/* Mark this spot as free */
	plot_XY_popup_widget[i] = (Widget)(-1);
	n_plot_xy--;
	n_lines_on_plot[i] = 0;
	plot_XY_locked[i]  = FALSE;
}

	int
locked_plot()
{
	int	i;

	for( i=0; i<MAX_PLOT_XY; i++ )
		if( plot_XY_locked[i] == TRUE ) 
			return( i );
	
	return( -1 );
}
